@@ -2,6 +2,7 @@ import time
 import pathlib
 from email_handler import GMailAcc, DriveAcc, SheetsAcc
 from httplib2.error import ServerNotFoundError
+from googleapiclient.errors import HttpError
 import os
 
 # todo ensure sensor and other errors work properly - sensor disconnect as warning emails? ensure controller keeps running
@@ -108,15 +109,18 @@ class Controller:
         """ typical controller run sequence - frequency in seconds [s]"""
 
         while True:
-            self.update_status(freq)
-            self.check_data_requests()
+            try:
+                self.update_status(freq)
+                self.check_data_requests()
 
-            # do shit
+                # do shit
 
-            self.deliver_warnings()
-            self.deliver_alerts()
+                self.deliver_warnings()
+                self.deliver_alerts()
 
-            time.sleep(freq)
+                time.sleep(freq)
+            except HttpError:
+                pass
 
     def check_data_requests(self):
         """ check for data requests and reply with data if desired """
@@ -382,4 +386,6 @@ if __name__ == '__main__':
 
     # create warnings
 
-    controller.deliver_warnings()
+    controller.check_data_requests()
+
+
